import { writable } from "svelte/store";
import { Router, parse } from "./vendor";
import {
  ROOT_URL,
  HASHCHANGE,
  navigateTo,
  cleanPath,
  isActive,
  router
} from "./utils";
const baseRouter = new Router();
const routeInfo = writable({});
const onError = {};
const shared = {};
let errors = [];
let routers = 0;
let interval;
let currentURL;
router.subscribe((value) => {
  shared.router = value;
});
routeInfo.subscribe((value) => {
  shared.routeInfo = value;
});
function doFallback(failure, fallback) {
  routeInfo.update((defaults) => ({
    ...defaults,
    [fallback]: {
      ...shared.router,
      failure
    }
  }));
}
function handleRoutes(map, params) {
  const keys = [];
  map.some((x) => {
    if (x.key && x.matches && !shared.routeInfo[x.key]) {
      if (x.redirect && (x.condition === null || x.condition(shared.router) !== true)) {
        if (x.exact && shared.router.path !== x.path)
          return false;
        navigateTo(x.redirect);
        return true;
      }
      if (x.exact) {
        keys.push(x.key);
      }
      Object.assign(params, x.params);
      routeInfo.update((defaults) => ({
        ...defaults,
        [x.key]: {
          ...shared.router,
          ...x
        }
      }));
    }
    return false;
  });
  return keys;
}
function evtHandler() {
  let baseUri = !HASHCHANGE ? window.location.href.replace(window.location.origin, "") : window.location.hash || "/";
  let failure;
  if (ROOT_URL !== "/") {
    baseUri = baseUri.replace(cleanPath(ROOT_URL), "");
  }
  if (/^#[\w-]+$/.test(window.location.hash) && document.querySelector(window.location.hash) && currentURL === baseUri.split("#")[0])
    return;
  const [fixedUri, qs] = baseUri.replace("/#", "#").replace(/^#\//, "/").split("?");
  const fullpath = fixedUri.replace(/\/?$/, "/");
  const query = parse(qs);
  const params = {};
  const keys = [];
  routeInfo.set({});
  if (currentURL !== baseUri) {
    currentURL = baseUri;
    router.set({
      path: cleanPath(fullpath),
      query,
      params
    });
  }
  baseRouter.resolve(fullpath, (err, result) => {
    if (err) {
      failure = err;
      return;
    }
    keys.push(...handleRoutes(result, params));
  });
  const toDelete = {};
  if (failure && failure.path !== "/") {
    keys.reduce((prev, cur) => {
      prev[cur] = null;
      return prev;
    }, toDelete);
  } else {
    failure = null;
  }
  errors.forEach((cb) => cb());
  errors = [];
  try {
    baseRouter.find(cleanPath(fullpath)).forEach((sub) => {
      if (sub.exact && !sub.matches) {
        toDelete[sub.key] = null;
      }
    });
  } catch (e) {
  }
  routeInfo.update((defaults) => ({
    ...defaults,
    ...toDelete
  }));
  let fallback;
  Object.keys(onError).forEach((root) => {
    if (isActive(root, fullpath, false)) {
      const fn = onError[root].callback;
      fn(failure);
      errors.push(fn);
    }
    if (!fallback && onError[root].fallback) {
      fallback = onError[root].fallback;
    }
  });
  if (failure && fallback) {
    doFallback(failure, fallback);
  }
}
function findRoutes() {
  clearTimeout(interval);
  interval = setTimeout(evtHandler);
}
function addRouter(root, fallback, callback) {
  if (!routers) {
    window.addEventListener("popstate", findRoutes, false);
  }
  if (!onError[root] || fallback) {
    onError[root] = { fallback, callback };
  }
  routers += 1;
  return () => {
    routers -= 1;
    if (!routers) {
      window.removeEventListener("popstate", findRoutes, false);
    }
  };
}
export {
  addRouter,
  baseRouter,
  doFallback,
  evtHandler,
  findRoutes,
  handleRoutes,
  routeInfo
};
